--  Telemetry script: RES Training App
--  #################################
--
--  version 11   15.08.2021
--  by ChrisOHara
--  written on OpenTX version 2.3.7
--  for Horus and Taranis radios (but may run on other openTX radios)
--  This Software is free
--
--  Information: Using a barometer this software is not allowed in official RES competion.
--
--  Requirements: Telemetry value "Alt" (so a barometer is required)
--                Momentary switch "sf" and "sg"
--                   (switches can be changed by altering software definitions, switchname_one and switchname_two)
-- 
--
--  Reaching 5m (starting height) you will hear a beep (1000 Hz).
--  During the following 10 seconds you can start the timer by operating momentary switch "sf" or "sg"
--     when plane is released from leash.
--  If you do not operate a switch  the  timer will start after 10 seconds on its own.
--  When timer starts you can hear a beep (2000Hz).
--
--  reaching landing height (2m) there is another beep (1000Hz).
--  During the next 10 seconds you can stop the timer by operating momentary switch "sf" or "sg"
--     when having contact with ground.
--  If you do not operate a switch the timer timer will stop after 10 seconds on its own.
--  When timer stops you can hear a beep (2000Hz).
--
--  Hint: If you do not use a swich  you will get the flighttime between 
--        5m (starting altidude) and 2m (landing altidude).
--
--  The script does use/define timer 1 on its own. (6 minutes, countdown)
--
--  There will be displayed the last flights. 
--      Horus: max is 2,6,8,19 depending on screen zone you select for widget
--      Taranis: max is 8
--
--  Information displayed:
--      Flight     =    Start-time of flight hour:minutes (newest flight always on top)
--      Time       =    Flight-time in minutes:seconds
--      Alt-S      =    starting altidude
--      Alt-F      =    maxmal altidude (during flight from start timer to stop timer)
--
--  New features in Version 11:
--    1. If receiver is switch off you get a 3s beep and a warning on the screen "not armed".
--       If you don't like that you search the two lines starting with "-- armed-warning" at the end
--         and change the line to comment.
--       (I added this function after I successfully launching my plane without switching on the receiver. Haha)
--    2. The GR6 firmware (at least access version) has a firmware bug. When switching on the receiver the altitdue rises
--        from zero to geografical altitude. Therefore openTX function auto-offset does not work properly and also this
--        lua script wrongly started a flight. Now the script does reset the telemetry value "Alt" (and "VSpd") three seconds 
--        after the receiver is switched on. But this works only for OpenTx 2.3.11 or higher.
--        For older releases there is only one solution: Do switch on the receiver first and then switch on the transmitter.
--        Or select the model after switching on the receiver.
--    3. In the Horus version the code was moved from the telemetry script to the widget (telemetry script is a dummy now)
--
--  Installation:
--      Horus: Copy /WIDGETS/res_H/main.lua and /SCRIPTS/TELEMETRY/dummy.lua   to the sd_card of your Horus.
--             Add the widget res_H to your telemetry screen.
--      Taranis: Copy /SCRIPTS/TELEMETRY/res_T.lua   to the sd_card of your Taranis.
--               Add the script res_T to your telemetry screen.
--
--  Errors: Scripts aborts when function getValue() is called with nil.
--  reason: Telemetry value "Alt" is not available or
--          one or both switches "sf" or "sg" are not available.(change script, define other switches
--          in "switchname_one" and/or "switchname_two")
--
-- 
--  Telemetry Script:  Trainingsprogramm für RES Flieger
--  ####################################################
--
--  version 11   15.08.2021
--  von ChrisOHara
--  geschrieben mit OpenTX version 2.3.7
--  für Horus and Taranis radios (läuft aber eventuell auch auf anderen openTX Radios)
--  diese Software ist frei
--
--  Hinweis: Da ein Barometer verwendet wird, ist die Software nicht in einem RES Wettbewerb erlaubt.
--
--  Voraussetzungen:
--           Benötigt wird der Telemetriewert "alt"  (Höhe/Barometer).
--           Momentary Schalter "sf" oder "sg"
--             (kann im script verändert werden über switchname_one und switchname_two)
--  
--  Bei 5m (Starthöhe) ertönt ein Beep (1000 Hz).
--  Innerhalb der nächsten zehn Sekunden kann man durch Drücken der Tasten "sf" 
--    oder "sg" (beim Ausklinken des Seils) den Timer starten.
--  Drückt man keine Taste, wird der Timer automatisch nach zehn Sekunden gestartet.
--  Wenn der Timer startet, ertönt ein Beep (2000 Hz).
--  
--  Bei 2m (Landehöhe) ertönt wieder ein Beep (1000 Hz).
--  Innerhalb der nächsten zehn Sekunden kann man (bei Bodenberührung) durch Drücken der Tasten "sf" 
--    oder "sg" den Timer stoppen.
--  Drückt man keine Taste, wird der Timer automatisch nach zehn Sekunden gestoppt.
--  Wenn der Timer stopped, ertönt ein Beep (2000 Hz).
--  
--  Hinweis: Verzichtet man auf das Drücken der Tasten erhält man demzufolge die Zeit zwischen 
--           Starthöhe (5m) und Landehöhe (2m).
--  
--  Das Programm verwendet/definiert automatisch den Timer 1. (6 Minuten, countdown)
--
--  Es werden die Daten der letzen Flüge angezeigt. 
--      Horus: maximal 2,6,8,19 abhängig vom gewählen Bildschirmausschnitt für das Widget
--      Taranis: maximal 8
--
--  Angezeigte Infomation:
--	Flight    =    Startzeit des Fluges (der jüngste Flug ist immer 0)
--	Time      =    Flugzeit in Minuten:Sekunden
--      Alt-S     =    Starthöhe (starten des Timers)
--      Alt-F     =    maximale Flughöhe (während des gesamten Fluges zwischen Timer-Start und Timer-Stop)
--
--  Neues in Version 11:
--    1. Wenn der Empfänger ausgeschaltet ist, ertönt ein 3s Beep und es erscheint eine Warnung auf dem Display "not armed".
--       Wenn das nicht gefällt, nach "-- armed-warning" suchen und die Zeile auf Kommentar setzen.
--       (Funktion kam dazu, nachdem ich meinen Flieger mit ausgeschalteten Empfänger hoch geschossen habe. Haha)
--    2. Die GR6 Firmware (zumindest Access Variante) hat einen Fehler. Nach dem Einschalten des Empfängers steigt die Höhe
--        Null auf die geografische Höhe kontinuierlich an. Das führt dazu, dass zB. Auto-Offset in OpenTx nicht richtig funktioniert
--        und dieses LUA Skript fälschlicherweise einen Flug startet. Das Skript macht nun einen Reset des Telemetriewerts "Alt" (und "VSpd").
--        drei Sekunden nach dem Einschalten des Empfängers. Leider funktioniert der Workaround nur für Opentx 2.3.11 und höher.
--        Für älterer Versionen hilft nur: Zuerst Empfänger einschalten und dann erst den Sender.
--        Oder das Modell erst nach dem Einschalten des Empfängers auswählen.
--    3. In der Horusvariante wurde der Code aus dem Telemetrie Script in das Widget verschoben (Telemetrie Script ist jetzt ein Dummy)
--
--  Installation:
--      Horus: Kopiere /WIDGETS/res_H/main.lua und /SCRIPTS/TELEMETRY/dummy.lua auf die sd_card deiner Horus.
--             Füge das Widget res_H deinem Telemetry Screen hinzu.
--      Taranis: Kopiere /SCRIPTS/TELEMETRY/res_T.lua   auf die sd_card deiner Taranis.
--               Füge das Script res_T deinem Telemetry Screen hinzu.               
--
--  Fehler: Das Script bricht ab, wenn die Funktion getValue() mit "nil" aufgerufen wird.
--  Grund:  Der Telemetrie Wert "Alt" ist nicht verfügbar oder
--          einen oder beide Schalter "sf" oder "sg" gibt es nicht.(Script ändern, in "switchname_one"
--          und "switchname_two" andere Schalter eintragen) 
--
--  #######################################################################################################


local my_start_altitude = 5;       -- the altitude to recognize starting condition in [m]
local landing_altitude = 2;        -- the altitude to recognize landing condition in [m]
local start_stop_time_delta = 10;
local my_timer_number = 0;	   -- the timer that is resetted by start condition

local switch_condition_start = -1024;
local switch_condition_stop = -1024;
local switchname_one = "sf";
local switchname_two = "sg";
local switchId_one = nil;
local switchId_two = nil;
local my_timer = nil;
local start_time;
local flight_time_start_end_limit;
local altitude = 0;
local max_altitude = 0;
local altId = nil;
local my_condition = 0;          -- 0="looking for start", 1="looking for flying"; 2="looking for landing"; 3="looking for stop timer"

local max_flight_counter = 8;    -- maximum number of flights stored, prevents screen overflow
local flight_count = 0;          -- counts flights stored
local i=0;
local my_minutes = 0;
local my_seconds = 0;
local flight_time_0 = 0;
local flight_start_times = {};
local flight_times = {};
local start_altitudes = {};
local flight_altitudes = {};

local my_armed = 100;
local my_sensor_reset_time = 0;
 
local buildMode = ...
local res = nil

local options = {
	{ "Restore", BOOL, 0},
	{ "Text", COLOR, BLACK},
	{ "Warning", COLOR, YELLOW}
}
local TELE_PATH = "/SCRIPTS/TELEMETRY/"



-- ############################# create/init function ##################
local function res_H_create(zone, options)

    altId = getFieldInfo("Alt") and getFieldInfo("Alt").id or nil;
    switchId_one = getFieldInfo(switchname_one) and getFieldInfo(switchname_one).id or nil;  --  defines switch as switch for time meassurement
    switchId_two = getFieldInfo(switchname_two) and getFieldInfo(switchname_two).id or nil;  --  defines switch as switch for time meassurement
    my_timer = model.getTimer(my_timer_number);
      my_timer.mode = 0;
      my_timer.start = 360;
      my_timer.value = 360;
      my_timer.countdownBeep = 2;
      my_timer.minuteBeep = false;
      my_timer.persistant = 0;
    model.setTimer(my_timer_number , my_timer);

    resZone = { zone = zone, options = options }
    res = loadScript(TELE_PATH .. "dummy.lua","bt")()
    max_flight_counter = math.floor(resZone.zone.h / 10) -1 ;
    return resZone
end

-- #################### This function allow updates when you change widgets settings #####################
local function update(resZone, options)
	resZone.options = options
	return
end


-- ############################# looking for armed ##################
local function look_for_armed()           -- called when condition is 0
  if my_armed < 2 then 
       playTone(800, 3000, 6000, PLAY_NOW ) ;
     else
       my_sensor_reset_time = getTime() + 300;       -- delta time to reset is 300*0.01 seconds
       my_condition = 1;
     end
end

-- ############################ lookig for reset of sensor ###############################
local function look_for_reset_sensor()     

   if getTime() < my_sensor_reset_time then                -- called as long as my_condition = 1
      return;
      end

   my_condition = 2;

   local ver, radio, maj, minor, rev = getVersion()       -- finding version of OTx
   local my_version =maj*10000 + minor*100 + rev ;
   if my_version < 20311 then 
        return ;
      end

    for i = 0, 31 do                                      -- finding sensorID Alt and VSpd and reset
      local name = model.getSensor(i).name;
      if name == "Alt" then
        model.resetSensor(i);                             -- does need openTx 2.3.11 or higher
        end
      if name == "VSpd" then
        model.resetSensor(i);                             -- does need openTx 2.3.11 or higher
        end
      end

end
     
  

-- #############################  looking for start #############
local function look_for_start()                                     -- called as long as my_condition is 2
   altitude=getValue(altId);
   if altitude > my_start_altitude then 
     max_altitude = altitude;
     for i = (max_flight_counter-1),1,-1 do    -- shift Globalvariables to empty first slot for newer values
        flight_start_times [i] = flight_start_times [i-1];
        flight_times [i] = flight_times [i-1];
        start_altitudes [i] = start_altitudes [i-1];
        flight_altitudes [i] = flight_altitudes [i-1];
       end
     max_altitude = 0;
     start_altitudes[0]=0;
     flight_start_times[0] = "00:00";
     flight_times[0] = "0:00";
     flight_altitudes[0] = 0;
     if flight_count < max_flight_counter then
        flight_count = flight_count + 1;
       end
     flight_time_start_end_limit = getTime() + start_stop_time_delta*100;
     switch_condition_start = -1024; 
     switch_condition_stop  = -1024;
     playTone(1000, 500, 500, PLAY_NOW); 
     my_condition = 3;
   end
end

-- #############################  looking for flying (release from leach) ############
local function look_for_flying()                                   -- called as long as my_condition is 3

   altitude=getValue(altId);
   if altitude > max_altitude then 
	max_altitude = altitude;
        start_altitudes[0] = (math.floor(max_altitude*10 + 0.5)) /10;
	flight_altitudes[0] = start_altitudes[0];
     end

   start_time=getTime();

   if ((getValue(switchId_one) == 1024) or (getValue(switchId_two) == 1024)) then
	switch_condition_start=1024;
     end
   if ((start_time >= flight_time_start_end_limit) or (switch_condition_start == 1024) ) then
        local my_date_time = getDateTime();
	flight_start_times[0] = string.format("%02d:%02d", my_date_time.hour, my_date_time.min);
        my_timer.mode = 1;
        model.setTimer(my_timer_number , my_timer);
	playTone(2000, 500, 500, PLAY_NOW);
	my_condition = 4;
     end
   return
end

-- #############################  looking for landing ############
local function look_for_landing()                                   -- called as long as my_condition is 4

   altitude=getValue(altId);
   if altitude > max_altitude then 
	max_altitude = altitude;
        flight_altitudes[0] = (math.floor(max_altitude*10 + 0.5)) /10;
     end

   flight_time_0 = math.floor((getTime() - start_time) / 100 + 0.5);
   my_minutes = math.floor(flight_time_0 / 60);
   my_seconds = flight_time_0 - (my_minutes * 60);
   flight_times[0] = string.format("%d:%02d", my_minutes, my_seconds);

   if altitude < landing_altitude then
        flight_time_start_end_limit = flight_time_0 + start_stop_time_delta;
	playTone(1000, 500, 500, PLAY_NOW);
	my_condition = 5;
     end
   return
end

-- ############################# looking for stop timer ###########
local function look_for_stop_timer()                                 -- called as log as my_condition is 5

   flight_time_0 = math.floor((getTime() - start_time) / 100 + 0.5);
   my_minutes = math.floor(flight_time_0 / 60);
   my_seconds = flight_time_0 - (my_minutes * 60);
   flight_times[0] = string.format("%d:%02d", my_minutes, my_seconds);

   if ( (getValue(switchId_one) == 1024) or (getValue(switchId_two) == 1024) ) then
	switch_condition_stop=1024; 
     end
   if ((flight_time_0 >= flight_time_start_end_limit) or (switch_condition_stop == 1024) ) then
        my_timer.mode = 0;
        model.setTimer(my_timer_number , my_timer);
	playTone(2000, 500, 500, PLAY_NOW);
	my_condition = 2;
     end
   return
end

-- ############################### background function #################
local function res_H_background(resZone)
  my_armed = getRSSI() ;
  if my_armed < 2 then 
      my_condition = 0;
      end;
  if       my_condition == 0 then look_for_armed();  
    elseif my_condition == 1 then look_for_reset_sensor();
    elseif my_condition == 2 then look_for_start();
    elseif my_condition == 3 then look_for_flying();
    elseif my_condition == 4 then look_for_landing();
    elseif my_condition == 5 then look_for_stop_timer();
    end
  return
end


-- ############# refresh/run Called periodically when custom telemetry screen containing widget is visible. #################
local function res_H_refresh(resZone)

    res_H_background();

    lcd.drawText (resZone.zone.x, resZone.zone.y-5, "Flight Time    Alt-S    Alt-F   ", SMLSIZE+INVERS);
    if flight_count > 0 then
        local yy = resZone.zone.y + 10
        if my_condition < 3 then
	      lcd.drawText (resZone.zone.x,yy,flight_start_times[0], SMLSIZE);
             else
              lcd.drawText (resZone.zone.x,yy,"flying", SMLSIZE);
             end
        lcd.drawText (resZone.zone.x+45,yy,flight_times[0], SMLSIZE)
        lcd.drawText (resZone.zone.x+95,yy,start_altitudes[0], SMLSIZE)
        lcd.drawText (resZone.zone.x+145,yy,flight_altitudes[0], SMLSIZE)
        end

    for i=1,(flight_count-1) do
	yy = resZone.zone.y + 10 + i*10;
	lcd.drawText (resZone.zone.x,yy,flight_start_times[i], SMLSIZE)
	lcd.drawText (resZone.zone.x+45,yy,flight_times[i], SMLSIZE)
	lcd.drawText (resZone.zone.x+95,yy,start_altitudes[i], SMLSIZE)
	lcd.drawText (resZone.zone.x+145,yy,flight_altitudes[i], SMLSIZE)
        end
    if my_condition == 0 then  lcd.drawText (resZone.zone.x+30,resZone.zone.y+25,"not armed", MIDSIZE+INVERS); end  -- armed-warning
    return
end


return { name = "Res_H", options=options, create=res_H_create, update=res_H_update, refresh=res_H_refresh, background=res_H_background}
